FileMeta

Struct FileMeta 

pub struct FileMeta {
Show 18 fields pub region_id: RegionId, pub file_id: FileId, pub time_range: FileTimeRange, pub level: Level, pub file_size: u64, pub max_row_group_uncompressed_size: u64, pub available_indexes: IndexTypes, pub indexes: Vec<ColumnIndexMetadata>, pub index_file_size: u64, pub index_version: u64, pub num_rows: u64, pub num_row_groups: u64, pub sequence: Option<NonZeroU64>, pub partition_expr: Option<PartitionExpr>, pub num_series: u64, pub primary_key_min: Option<Bytes>, pub primary_key_max: Option<Bytes>, pub preserve_row_sequence: bool,
}
Metadata of a SST file.

Fields

region_id: RegionId

Region that created the file. The region id may not be the id of the current region.

file_id: FileId

Compared to normal file names, FileId ignore the extension

time_range: FileTimeRange

Timestamp range of file. The timestamps have the same time unit as the data in the SST.

level: Level

SST level of the file.

file_size: u64

Size of the file.

max_row_group_uncompressed_size: u64

Maximum uncompressed row group size of the file. 0 means unknown.

available_indexes: IndexTypes

Available indexes of the file.

indexes: Vec<ColumnIndexMetadata>

Created indexes of the file for each column.

This is essentially a more granular, column-level version of available_indexes, primarily used for manual index building in the asynchronous index construction mode.

For backward compatibility, older FileMeta versions might only contain available_indexes. In such cases, we cannot deduce specific column index information from available_indexes alone. Therefore, defaulting this indexes field to an empty list during deserialization is a reasonable and necessary step to ensure column information consistency.

index_file_size: u64

Size of the index file.

index_version: u64

Version of the index file. Used to generate the index file name: “{file_id}.{index_version}.puffin”. Default is 0 (which maps to “{file_id}.puffin” for compatibility).

num_rows: u64

Number of rows in the file.

For historical reasons, this field might be missing in old files. Thus the default value 0 doesn’t means the file doesn’t contains any rows, but instead means the number of rows is unknown.

num_row_groups: u64

Number of row groups in the file.

For historical reasons, this field might be missing in old files. Thus the default value 0 doesn’t means the file doesn’t contains any rows, but instead means the number of rows is unknown.

sequence: Option<NonZeroU64>

File-level sequence bound or admission marker in the target region.

Flush records the maximum input row sequence. Compaction outputs inherit the maximum input bound, or remain unknown if any input bound is unknown, independently of per-row sequence trust. This does not imply that every physical row has this sequence. Readers also use it to normalize foreign and legacy all-zero files.

partition_expr: Option<PartitionExpr>

Partition expression from the region metadata when the file is created.

This is stored as a PartitionExpr object in memory for convenience, but serialized as JSON string for manifest compatibility. Compatibility behavior:

  • None: no partition expr was set when the file was created (legacy files).
  • Some(expr): partition expression from region metadata.
num_series: u64

Number of series in the file.

The number is 0 if the series number is not available.

primary_key_min: Option<Bytes>

Minimum primary key value in the file, encoded as bytes. None if the primary key range is not available (e.g., legacy files).

primary_key_max: Option<Bytes>

Maximum primary key value in the file, encoded as bytes. None if the primary key range is not available (e.g., legacy files).

preserve_row_sequence: bool

Whether the file preserves per-row sequence numbers usable for exact row-level sequence filtering. Merely retaining physical input sequences during compaction does not restore this capability for untrusted inputs.

Implementations

Source

impl FileMeta

pub fn primary_key_range(&self) -> Option<(Bytes, Bytes)>

Returns the primary key range if both min and max are present.

pub fn exists_index(&self) -> bool

pub fn index_version(&self) -> Option<IndexVersion>

pub fn is_index_up_to_date(&self, other: &FileMeta) -> bool

Whether the index file is up-to-date comparing to another file meta.

pub fn inverted_index_available(&self) -> bool

Returns true if the file has an inverted index

pub fn fulltext_index_available(&self) -> bool

Returns true if the file has a fulltext index

pub fn bloom_filter_index_available(&self) -> bool

Returns true if the file has a bloom filter index.

pub fn index_file_size(&self) -> u64

pub fn is_index_consistent_with_region( &self, metadata: &[ColumnMetadata], ) -> bool

Check whether the file index is consistent with the given region metadata.

pub fn file_id(&self) -> RegionFileId

Returns the cross-region file id.

pub fn index_id(&self) -> RegionIndexId

Returns the RegionIndexId for this file.